<?php
/**
 * Plugin Name: Gallery Made Easy
 * Plugin URI: https://wordpress.org/plugins/gallery-made-easy/
 * Description: Instantly create responsive image galleries within a simple and customizable interface. Just upload, add, organise and publish.
 * Version: 1.3
 * Author: Earl Evan Amante
 * Author URI: https://ph.linkedin.com/in/earlamante
 * License: GPL2
 */
	/**
	 * Common fix for adding menu page in plugins
	 */
	include(ABSPATH . "wp-includes/pluggable.php"); 
	
	Class w3b_gallery {
		var $plugin_name = 'Gallery Made Easy';
		var $menu_title = 'Settings';
		var $menu_slug	= 'w3b-gallery';
		var $post_type = 'w3b_gallery';
		var $plugin_description = '
			<p>Instantly create responsive image galleries within a simple and customizable interface. Just upload, add, organise and publish.</p>
		';
		var $icon_url;
		var $in_page;

		var $roles = array(
			'Administrator'		=> 'manage_options',
			'Editor'			=> 'publish_pages',
			'Author'			=> 'publish_posts',
			'Contributor'		=> 'edit_posts',
			'Subscriber'		=> 'read',
		);
		var $statuses = array(
			'',
			'updated',
			'error'
		);
		var $status;
		var $msg;
		var $gallery_id;
		var $display_type;
		var $output_css;

		public function __construct() {
			$this->init();
		}

		private function _get_minimun_role() {
			return $this->roles['Editor'];
		}

		public function init() {
			global $post;
			$this->output_css = get_option( 'w3b_gallery_o', 1 );

			add_action( 'admin_init', array($this, 'get_msg') );

			// Check if regenerate thumbnail is activated
			add_action( 'admin_init', array($this, 'check_regen_thumbnail') );

			// Admin css and js
			add_action( 'admin_menu', array($this, 'admin_menu') );
			add_action( 'admin_enqueue_scripts', array($this, 'admin_head') );
			add_action( 'admin_head', array($this, 'register_mce_button') );
			add_action( 'wp_enqueue_scripts', array($this, 'frontend_head') );

			// admin columns
			add_filter( 'manage_edit-'. $this->post_type .'_columns', array($this, 'admin_columns') ) ;
			add_action( 'manage_'. $this->post_type .'_posts_custom_column', array($this, 'admin_column_content'), 10, 2 );

			// admin metabox
			add_action( 'add_meta_boxes_' . $this->post_type, array($this, 'add_metabox') );

			// Trigger on save post
			add_action( 'save_post', array($this, 'save_gallery') );

			// Print frontend
			add_shortcode( $this->post_type, array($this, 'print_gallery') );
			
			// Image sizes
			$width = get_option( 'w3b_gallery_w', get_option( 'thumbnail_size_w', 150 ) );
			$height = get_option( 'w3b_gallery_h', get_option( 'thumbnail_size_h', 150 ) );
			add_image_size( 'w3b_gallery', $width, 9999 );
			add_image_size( 'w3b_gallery_cropped', $width, $height, TRUE );

			// Custom Post Type
			add_action( 'init', array( $this, 'add_custom_post_type' ) );
		}
		
		public function check_regen_thumbnail() {
			if( !class_exists('RegenerateThumbnails') )
				add_action( 'admin_notices', array($this, 'check_regen_notice') );
		}

		public function check_regen_notice() {
			echo '
		<div class="warning notice notice-warning">
			<h3>Gallery Made Easy warning:</h3>
			<p>For best experience you will need to regenerate the image thumbnails especially when you update the thumbnail size in the settings.</p>
			<p>Please install <a href="https://wordpress.org/plugins/regenerate-thumbnails/" target="_blank">Regnerate Thumbnails Plugin</a></p>
		</div>
			';
		}

		public function add_custom_post_type() {
			register_post_type( $this->post_type,
				array(
					'labels' => array(
						'name' => __( 'Galleries' ),
						'singular_name' => __( 'Gallery' )
					),
					'public' => true,
					'menu_icon'	=> plugins_url( 'js/icon.png', __FILE__ ),
					'supports'	=> array( 'title', 'thumbnail' )
				)
			);
		}	
		
		public function admin_menu() {
			add_submenu_page(
				'edit.php?post_type=' . $this->post_type,
				$this->plugin_name,
				$this->menu_title,
				$this->_get_minimun_role(),
				$this->menu_slug,
				array($this, 'print_page')
			);
		}

		public function admin_head() {
			wp_register_style( 'jqui_css', plugins_url( 'css/jquery-ui.min.css', __FILE__ ), false, '1.3' );
			wp_register_style( 'jqui_structure_css', plugins_url( 'css/jquery-ui.structure.min.css', __FILE__ ), false, '1.3' );
			wp_register_style( 'jqui_theme_css', plugins_url( 'css/jquery-ui.theme.min.css', __FILE__ ), false, '1.3' );
			wp_register_style( 'w3b_custom_css', plugins_url( 'css/style.css', __FILE__ ), false, '1.3' );
			wp_enqueue_style( 'w3b_custom_css' );

			wp_enqueue_script( 'images-loaded', plugins_url( 'js/imagesloaded.pkgd.min.js', __FILE__ ), false, '1.3' );
			wp_enqueue_script( 'isotope', plugins_url( 'js/isotope.pkgd.min.js', __FILE__ ), false, '1.3', TRUE );
			wp_enqueue_script( 'w3b_custom_js', plugins_url( 'js/main.js', __FILE__ ), false, '1.3' );
		}

		public function register_mce_button() {
			if ( !current_user_can( 'edit_posts' ) && !current_user_can( 'edit_pages' ) )
				return;

			if ( 'true' == get_user_option( 'rich_editing' ) ) {
				add_filter( 'mce_external_plugins', array( $this, 'custom_tinymce_js' ) );
				add_filter( 'mce_buttons', array( $this, 'add_mce_button' ) );
			}

			$galls = get_posts(array(
				'post_type'			=> $this->post_type,
				'posts_per_page'	=> -1
			));

			if( $galls ) {
				$gals = array();

				foreach( $galls as $g )
					$gals[] = "{
                            text: '". $g->post_title ."',
                            value: '". $g->ID ."'
                        }";

				echo '
					<script>
						var sgallery = [
						'. implode(',', $gals) .'
						];
					</script>
				';
			}
		}

		public function custom_tinymce_js( $plugin_array ) {
			$plugin_array['w3b_gallery_button'] = plugins_url( '/js/mce_button.js', __FILE__ );
			return $plugin_array;
		}

		public function add_mce_button( $b ) {
			array_push( $b, 'w3b_gallery_button' );
			return $b;
		}

		public function frontend_head() {
			wp_register_style( 'lightgallery', plugins_url( 'js/light-gallery/css/lightgallery.min.css', __FILE__ ), false, '1.3' );
			wp_enqueue_style( 'lightgallery' );
			
			if( $this->output_css ) {
				wp_register_style( 'w3b-gallery', plugins_url( 'template/style.css', __FILE__ ), false, '1.3' );
				wp_enqueue_style( 'w3b-gallery' );
			}
			
			wp_enqueue_script( 'jquery-mousewheel-script', plugins_url( 'js/jquery.mousewheel.min.js', __FILE__ ), array('jquery-core'), '1.3', TRUE );
			wp_enqueue_script( 'lightgallery-core', plugins_url( 'js/light-gallery/js/lightgallery.min.js', __FILE__ ), array('jquery-mousewheel-script'), '1.3', TRUE );
			wp_enqueue_script( 'lightgallery-thumbnail', plugins_url( 'js/light-gallery/js/lg-thumbnail.min.js', __FILE__ ), array('lightgallery-core'), '1.3', TRUE );
			wp_enqueue_script( 'lightgallery-fullscreen', plugins_url( 'js/light-gallery/js/lg-fullscreen.min.js', __FILE__ ), array('lightgallery-core'), '1.3', TRUE );

			wp_enqueue_script( 'images-loaded', plugins_url( 'js/imagesloaded.pkgd.min.js', __FILE__ ), false, '1.3' );
			wp_enqueue_script( 'isotope', plugins_url( 'js/isotope.pkgd.min.js', __FILE__ ), false, '1.3', TRUE );
			wp_enqueue_script( 'w3b-gallery-masonry', plugins_url( 'template/masonry.js', __FILE__ ), false, '1.3', TRUE );
			
			wp_enqueue_script( 'w3b-gallery', plugins_url( 'template/main.js', __FILE__ ), false, '1.3', TRUE );
		}

		public function add_metabox() {
			global $post;
			$this->gallery_id = $post->ID;

			add_meta_box(
				$this->menu_slug,
				$this->plugin_name,
				array($this, 'metabox'),
				NULL,
				'advanced',
				'high'
			);

			add_meta_box(
				$this->menu_slug . '_side',
				$this->plugin_name . ' Display Type',
				array($this, 'metabox_side'),
				NULL,
				'side',
				'high'
			);

			add_action( 'edit_form_after_title', array( $this, 'metabox_after_title') );
		}

		public function metabox_after_title() {
		    global $post, $wp_meta_boxes;
		    do_meta_boxes( get_current_screen(), 'advanced', $post );
		    unset( $wp_meta_boxes[get_post_type($post)]['advanced'] );
		}

		public function metabox( $post ) {
			echo $this->get_template( 'metabox', array(
				'post'	=> $post
			) );
		}

		public function metabox_side( $post ) {
			echo $this->get_template( 'metabox_side', array(
				'post'	=> $post
			) );
		}

		public function admin_columns( $columns ) {

			$columns = array(
				'cb' => '<input type="checkbox" />',
				'title' => __( 'Gallery Name' ),
				'shortcode' => __( 'Shortcode' ),
				'date' => __( 'Date' )
			);

			return $columns;
		}

		public function admin_column_content( $column, $post_id ) {
			global $post;

			switch( $column ) {
				case 'shortcode':
									echo '['. $this->post_type .' id="'. $post_id .'"]';
									break;
			}
		}

		public function get_msg() {
			if( $this->msg )
				add_action( 'admin_notices', function() {
					echo '<div id="message" class="'. $this->statuses[$this->status] .' notice is-dismissible below-h2"><p>'.$this->msg.'</p></div>';
				} );
		}

		public function cleanup($text='') {
			$text = preg_replace('/[^\w ]+/', '', strtolower($text) );
			return $text = preg_replace('/[ ]+/', '_', $text );
		}

		public function print_page() {
			if( !empty( $_POST ) ) {
				$this->update_option( 'w3b_gallery_w', $_POST );
				$this->update_option( 'w3b_gallery_h', $_POST );
				$this->update_option( 'w3b_gallery_o', $_POST );

				$this->status = 1;
				$this->msg = 'Settings updated.';
				$this->output_css = $display = get_option( 'w3b_gallery_o', TRUE );
			}
			echo $this->get_template( 'settings' );
		}

		private function update_option( $option_name, $data ) {
			if( isset( $data[$option_name] ) ) {
				update_option( $option_name, $data[$option_name] );
				return TRUE;
			}
			return FALSE;
		}

		// Main functionality
		// Check Section
			

		// Set Section
		public function save_gallery( $post_id ) {
			if( !empty( $_REQUEST['w3b_gallery_d'] ) ) {
				$gallery = json_decode( $_REQUEST['w3b-gallery__images'] );
				$d = $_REQUEST['w3b_gallery_d'];

				update_post_meta( $post_id, '_w3b_gallery', serialize( $gallery ) );
				update_post_meta( $post_id, '_w3b_gallery_d', $d );
			}
		}

		// Get Section
		public function get_template( $template_file='', $data=array() ) {
			if( !empty( $data ) ) extract( $data );
			if( file_exists( dirname( __FILE__ ) . '/views/' . $template_file . '.php' ) ) {
				ob_start();
				include( dirname( __FILE__ ) . '/views/' . $template_file . '.php' );
				return ob_get_clean();
			}
			return FALSE;
		}

		public function get_frontend( $template_file='', $data=array() ) {
			if( !empty( $data ) ) extract( $data );

			if( file_exists( get_template_directory() . '/w3b-gallery/' . $template_file . '.php' ) )
				$template = get_template_directory() . '/w3b-gallery/' . $template_file . '.php';
			elseif( file_exists( dirname( __FILE__ ) . '/template/' . $template_file . '.php' ) )
				$template = dirname( __FILE__ ) . '/template/' . $template_file . '.php';

			if( $template ) {
				ob_start();
				include( $template );
				return ob_get_clean();
			}
			return FALSE;
		}

		public function print_gallery( $atts ) {
			$p = shortcode_atts( array(
			    'id' => FALSE
			), $atts );

			$gal = get_post( $p['id'] );

			if( $p['id'] ) {
				$this->gallery_id = $p['id'];
				return $this->get_frontend( 'template', array(
					'w3b_gallery'	=> get_post_meta( $gal->ID, '_w3b_gallery', TRUE ),
				) );
			}

			return FALSE;
		}

		public function get_image_size() {
			if( $this->get_display_type() == 'masonry' )
				return 'w3b_gallery';
			else
				return 'w3b_gallery_cropped';
		}

		public function get_display_type() {
			if( !$this->display_type ) {
				$d = get_post_meta( $this->gallery_id, '_w3b_gallery_d', TRUE );
				$this->display_type = !empty( $d )? $d:'cropped';
			}

			return $this->display_type;
		}
		
		

		// Helper Functions
		
	}

	$w3b_gallery = new w3b_gallery;
?>